@@ -1,6 +1,7 @@
 """
 Author Babak.EA
 
+
 """
 
 import pandas as pd
@@ -16,6 +17,9 @@ import warnings
 warnings.filterwarnings("ignore")
 import os    
 from sklearn.preprocessing import MinMaxScaler  
+from random import random,choice
+import matplotlib.pyplot as plt
+
     
 def Path(path):
     if path[-1]=="/":
@@ -44,7 +48,7 @@ class Model_Generator:
     def __init__(self,df_C_Per_Product,df_C_NPS_Product,df_C_NPS_Extend,ProductCode="ProductCode",
                  Customer_ID="CustomerCIF",CustomerCount="CustomerCount",pr_category="SalesCategoryCornerstoneID",ResponseDate="ResponseDate",
                  StatedIncome="StatedIncome",CustomerScore="CustomerScore",
-                 path="./model/",limit=0.8,knn_neighbors=100):
+                 path="./model/",limit=0.79,knn_neighbors=100,Evaluate=True,Sample_size=0.03):
         """
         Model_Generator(
         "df_C_Per_Product=Dataset including Product information like Id, name, domain, popularity, count",
@@ -61,6 +65,7 @@ class Model_Generator:
             os.makedirs('model')
         if not os.path.exists('report'):
             os.makedirs('report')
+            
         
 
         self.report=dict()
@@ -81,6 +86,7 @@ class Model_Generator:
         self.df_C_Per_Product=df_C_Per_Product
         self.df_C_NPS_Product=df_C_NPS_Product
         self.df_C_NPS_Extend=df_C_NPS_Extend
+        self.Sample_size=Sample_size
 
         self.df_C_NPS_Product=df_C_NPS_Product.drop_duplicates(subset=[self.Customer_ID,self.ProductCode])# drope duplicates records
 
@@ -96,10 +102,31 @@ class Model_Generator:
         #print("ML ......")
         self.ML_model()
         self.ML_product_neighbors()
+        if Evaluate==True:
+            self.Evaluate()
     
 
         
     def product_dict(self):
+        """
+        Input:  "df_C_Per_Product=Dataset including Product information like Id, name, domain, popularity, count",
+        Out put: Dictionary  
+        "{00001000 ": {
+            "ProductCode": "00001000",
+            "ProductName": "Test Product",
+            "SalesCategoryCornerstoneID": "Test PR",
+            "CustomerCount": 9
+        },
+        {00002000 ": {
+            "ProductCode": "00002000",
+            "ProductName": "Test_2 Product",
+            "SalesCategoryCornerstoneID": "Test_2 PR",
+            "CustomerCount": 10
+        },
+        
+        the dict will be recorded as a PKL file in a Data folder(path) Pr_dict.pkl
+        """
+        
         #df_C_Per_Product
         self.Pr_dict=dict()
         self.Pr_dict=self.df_C_Per_Product.groupby(by=self.ProductCode, sort=True).apply(lambda x: x.to_dict(orient='records')).to_dict()
@@ -109,7 +136,18 @@ class Model_Generator:
         pickle.dump(self.Pr_dict, open(self.path+'Pr_dict.pkl','wb'))
 
     def Product_matrix(self):
-        #df=df.drop_duplicates(subset=[Customer_ID,ProductCode])# drope duplicates records
+        """
+        Input: "df_C_NPS_Products: user info , Income, and rating"
+        Output:
+        Product_hist.pkl: 
+        User Product Utility Matrix: its a sparce Product matrix
+        
+        Product_popularity.pkl:
+        User Product Binary Matrix 
+       
+        """
+        
+        
         self.Product_hist = self.df_C_NPS_Product.pivot(index=self.Customer_ID,
                                                         columns=self.ProductCode,
                                                         values=self.ProductCode).fillna(0)
@@ -120,6 +158,14 @@ class Model_Generator:
 
 
     def SVD_model(self):
+        """
+        Input: Product_popularity, the  User Product Binary Matrix which has been created through the (Product_matrix function)
+        output correlation matrix using PCA (SVD) with 50 Components
+        corr_mat.pkl 
+        """
+        
+        
+        
         X=self.Product_popularity.T
         SVD = TruncatedSVD(n_components=50, random_state=0) # the matrix will have 12 dimantion , dimandion reduction from 943 to 12 
         resultant_matrix = SVD.fit_transform(X)
@@ -128,6 +174,15 @@ class Model_Generator:
         pickle.dump(self.corr_mat, open(self.path+'corr_mat.pkl','wb'))
         pickle.dump(self.Product_list, open(self.path+'Product_list.pkl','wb'))
     def ML_model(self):
+        """
+        Input : Product_popularity Matrix which has been created through the (Product_matrix function)
+        Train the KNN Model using KNN user limit ( defaut =100) 
+        save the traing model as a KNN.pkl
+       
+        
+        """
+        
+        
         df=pd.merge(self.Product_popularity.reset_index(),self.df_C_NPS_Extend , on=self.CustomerCIF, how="left").fillna(0)
 
         df=df.set_index(self.CustomerCIF)
@@ -138,6 +193,12 @@ class Model_Generator:
 
         #return nbrs
     def ML_product_neighbors(self):
+        """
+        Input : df_C_Per_Product​:Dataset including Product information like Id, name, domain, popularity, count" 
+        Train the KNN Model using KNN user limit ( defaut =100) 
+        save the traing model as a PR_KNN.pkl
+        """
+        
         scaler = MinMaxScaler()
         Product_matrix =self.df_C_Per_Product.pivot_table(values=self.CustomerCount, index=self.ProductCode,
                                                                             columns=self.pr_category, fill_value=0)
@@ -148,13 +209,10 @@ class Model_Generator:
         
         
         X = self.product_df.iloc[:,:].values
-        nbrs = NearestNeighbors(n_neighbors=3).fit(X)
+        nbrs = NearestNeighbors(n_neighbors=2).fit(X)
         pickle.dump(nbrs, open(self.path+'PR_KNN.pkl','wb'))
         pickle.dump(self.product_df, open(self.path+'product_df.pkl','wb'))
-        
-    
-    
-    
+
     def User_info(self):
 
         self.df_C_NPS_Extend[[self.ResponseDate]] = self.df_C_NPS_Extend[[self.ResponseDate]].apply(pd.to_datetime)
@@ -180,12 +238,110 @@ class Model_Generator:
         del b
         
 
+
+    def Evaluate(self):
+
+        Income=[110000]
+        NPS=[0]
+        Sample = self.Product_hist.sample(frac=self.Sample_size)
+        Tem= {y:{"Product":{x for x in Sample.loc[y].tolist() if x !=0 } } for y in Sample.index.tolist() }#ok
+        #print(Tem)
+
+        del Sample
+        Acc_rep=dict()
+        Accuracy_rep=pd.DataFrame(columns=["Product","Product_count","USER","Popularity_based_suggestion",
+                                                         "User_similar","Products_neighbors","Flag"])
+
+        counter=0
+        for key in Tem.keys():
+            #print(key)
+            #print(Tem[key]["Product"])
+            pr_list=list(Tem[key]["Product"])
+            if len(pr_list) >=4:
+
+                Missing=choice(pr_list)                                
+
+                pr_list.remove(Missing)  
+                products=Product_Recommender("test",NEW_USER="True",USER_feature=pr_list+Income+NPS)
+                Reprt_list=[Missing]
+                #Peprt_list +=[test.Pr_dict[Missing][0]["CustomerCount"]]
+                Reprt_list +=[self.Pr_dict[Missing][0]["CustomerCount"]]
+                Reprt_list +=[(1 if Missing in y else 0) for y in [list(products.report[KEY].keys()) for KEY in list(products.report.keys())]]
+                Reprt_list+=[sum(Reprt_list[2:])] 
+
+                #print(Reprt_list)
+                Accuracy_rep.loc[counter]=Reprt_list
+                Reprt_list+=[1 if Missing in list(products.report["Popularity_based_suggestion"].keys()) else 0]
+
+                Popularity_based_suggestion=list(products.report["Popularity_based_suggestion"].keys())                 
+                User_similar=list(products.report["User_similar"].keys())                                                                   
+                #products_neighbors=list(products.report["products_neighbors"].keys())
+                counter +=1
+            else:
+                pass
+
+
+        Total_ACC=Accuracy_rep["Flag"].value_counts().to_dict()
+        PBS_ACC=Accuracy_rep["Popularity_based_suggestion"].value_counts().to_dict()
+        U_S_ACC=Accuracy_rep["User_similar"].value_counts().to_dict()
+        P_N_ACC=Accuracy_rep["Products_neighbors"].value_counts().to_dict()
+
+        Acc=pd.DataFrame(index=["Popularity_based_suggestion","User_similar","Products_neighbors","Total"],
+                                 columns=["Fail","Pass"],
+                         data=[
+                                [PBS_ACC[0],PBS_ACC[1]],
+                                  [U_S_ACC[0],U_S_ACC[1]],
+                                   [P_N_ACC[0],P_N_ACC[1]],
+                            [Total_ACC[0],Total_ACC[1]+Total_ACC[2]+Total_ACC[3]]])
+        print(Acc)
+        #Acc.plot.bar(stacked=True)
+
+        Totol_rec=len(Accuracy_rep)
+
+
+        def Totol_Acc():
+            #explode = (0, 0.0, 0.0,0.5)
+            Totoal_D={0:.5,1:0.0,2:0.0,3:0}
+            explode=[Totoal_D[x] for x in list(Total_ACC)]
+            #Totol+ACC_X=[Total_ACC[x]]
+            colors = ['darkorange', 'sandybrown', 'darksalmon', 'orangered','chocolate']
+            plt.pie(
+                list(Total_ACC.values()), 
+                labels=list(Total_ACC),
+                autopct='%.2f',
+                colors=colors,
+                explode=explode)
+
+
+            #plt.axis('equal')
+            plt.axis()
+        def Model_Acc(df):
+            display(df.plot.bar(stacked=True))
+
+
+
+
+        Totol_Acc()
+
+        Model_Acc(Acc)
+
+        self.model_report=Accuracy_rep
+        self.ACC_report=Acc
+        del Acc
+        del Accuracy_rep
+
+
+
+        
+        
+        
+
 #################################
 
 
 
 class Product_Recommender:
-    def __init__(self,USER_ID,Customer_ID="CustomerCIF",Model_path="./model/",Report_path="./report/",Limit=0.80,
+    def __init__(self,USER_ID,Customer_ID="CustomerCIF",Model_path="./model/",Report_path="./report/",Limit=0.79,
                  Local="True",
                  NEW_USER="False",USER_feature=[]):
         
@@ -287,12 +443,7 @@ class Product_Recommender:
         
         
         
-        """
-        
-        
-        
-        
-        
+        """        
         self.report=dict() # create the report dict 
         user_product=[]
         if self.UEW_USER=="False":
@@ -355,7 +506,7 @@ class Product_Recommender:
         
 
 
-            similar_product={key:val for key, val in similar_product.items() if val >= int(max_value*0.51)}
+            similar_product={key:val for key, val in similar_product.items() if val >= int(max_value*0.40)}
         except:
             similar_product={key:val for key, val in similar_product.items()}
         #print(similar_product)
@@ -372,14 +523,14 @@ class Product_Recommender:
                               val not in user_product])
         #print(self.suggestion)
 
-        self.suggestion={key:val for key, val in self.suggestion.items() if val >=round(len(user_product)/2.5) }
+        self.suggestion={key:val for key, val in self.suggestion.items() if val >=round(len(user_product)/8) }
         #Report["Popularity_based_suggestion"]=suggestion
         self.tem=[self.Pr_dict[x][0] for x in self.suggestion.keys()]
         
         for i in self.tem:
             self.suggestion[i["ProductCode"]]=i["CustomerCount"]
             
-        self.suggestion={k:v for k, v in self.suggestion.items() if v>=self.suggestion[max(self.suggestion, key=self.suggestion.get)]*.55}
+        self.suggestion={k:v for k, v in self.suggestion.items() if v>=self.suggestion[max(self.suggestion, key=self.suggestion.get)]* 0.40}
     
         #self.report["Popularity_based_suggestion"]=[self.Pr_dict[x][0] for x in self.suggestion.keys()]
 
